import sys
file_name = sys.argv[1]


import numpy as np
import matplotlib.pyplot as plt
import scipy.signal
import scipy.interpolate
import scipy.stats
import PIL.Image

def get_channel(img, a_off, b_off):
    step = 8
    img_chan = img[a_off::8, b_off::8]
    f = scipy.interpolate.interp2d(np.arange(a_off,img.shape[1],8), np.arange(b_off,img.shape[0],8), img_chan,
                                   kind = 'cubic')
    return f(np.arange(img.shape[1]), np.arange(img.shape[0]))

def get_green_channel(img):
    img_G1 = img[4::8, ::8]
    img_G2 = img[::8, 4::8]
    points1 = np.mgrid[4:img.shape[0]:8, :img.shape[1]:8].reshape((2,-1)).transpose()
    points2 = np.mgrid[:img.shape[0]:8, 4:img.shape[1]:8].reshape((2,-1)).transpose()
    points = np.concatenate((points1, points2))
    img_G = np.concatenate((img_G1.ravel(), img_G2.ravel()))
    a, b = np.mgrid[:img.shape[0], :img.shape[1]]
    return scipy.interpolate.griddata(points, img_G, (a, b), method = 'cubic')

def get_luminance_channel(img):
    img_Y1 = img[1::4, :]
    img_Y2 = img[2::4, :]
    img_Y3 = img[3::4, :]
    img_Y4 = img[:, 1::4]
    img_Y5 = img[:, 2::4]
    img_Y6 = img[:, 3::4]
    points1 = np.mgrid[1:img.shape[0]:4, :img.shape[1]].reshape((2,-1)).transpose()
    points2 = np.mgrid[2:img.shape[0]:4, :img.shape[1]].reshape((2,-1)).transpose()
    points3 = np.mgrid[3:img.shape[0]:4, :img.shape[1]].reshape((2,-1)).transpose()
    points4 = np.mgrid[:img.shape[0], 1:img.shape[1]:4].reshape((2,-1)).transpose()
    points5 = np.mgrid[:img.shape[0], 2:img.shape[1]:4].reshape((2,-1)).transpose()
    points6 = np.mgrid[:img.shape[0], 3:img.shape[1]:4].reshape((2,-1)).transpose()
    points = np.concatenate((points1, points2, points3, points4, points5, points6))
    img_Y = np.concatenate((img_Y1.ravel(), img_Y2.ravel(), img_Y3.ravel(), img_Y4.ravel(),
                            img_Y5.ravel(), img_Y6.ravel()))
    a, b = np.mgrid[:img.shape[0], :img.shape[1]]
    return scipy.interpolate.griddata(points, img_Y, (a, b), method = 'nearest')

def get_RGB(data, nbits, ncols = 1280):
    data = np.clip(data, 0, 2**nbits-1)
    img = data[:data.size//ncols*ncols].reshape((-1,ncols)).astype('float')
    # scale to [0,256)
    img *= 2**(8-nbits)

    # The sensor colour filter array is
    #  *-----------------*
    #  | B W W W G W W W |
    #  | W W W W W W W W |
    #  | W W W W W W W W |
    #  | G W W W R W W W |
    #  | W W W W W W W W |
    #  | W W W W W W W W |
    #  *-----------------*
    img_B = get_channel(img, 0, 0)
    img_R = get_channel(img, 4, 4)
    img_G = get_green_channel(img)
    img_Y = get_luminance_channel(img)

    # JPEG Y'CbCr conversion
    img_Cb = -0.168736*img_R-0.331264*img_G+0.5*img_B+128
    img_Cr = 0.5*img_R-0.418688*img_G-0.081312*img_B+128
    
    # CbCr translation for white calibration (determined experimentally with calibration image)
    Cb_translation = 14.47
    Cr_translation = -20.96
    img_Cb += Cb_translation
    img_Cr += Cr_translation

    img_R2 = img_Y + 1.402*(img_Cr-128)
    img_G2 = img_Y - 0.344136*(img_Cb-128) - 0.714136*(img_Cr-128)
    img_B2 = img_Y + 1.772*(img_Cb-128)

    img_RGB = np.stack((img_R2, img_G2, img_B2), axis = -1)
    return np.clip(np.round(img_RGB), 0, 255).astype('uint8'), img_Y, img_Cb, img_Cr


data3 = np.fromfile(file_name, dtype = 'int16')
img_data = get_RGB(data3, 12, ncols = 1408)[0]
PIL.Image.fromarray(img_data).save(file_name+'.jpg', 'JPEG');
